@@ -1,36 +1,49 @@
+from __future__ import annotations
+
 import asyncio
-from contextvars import ContextVar
 import itertools
+from collections.abc import Callable
+from contextvars import ContextVar, Token
+from types import TracebackType
+from typing import Self
 
 from .types import TaskGroupError
 
 __all__ = (
-    'TaskGroup',
-    'current_taskgroup',
+    "TaskGroup",
+    "TaskGroupError",
+    "current_taskgroup",
 )
 
-current_taskgroup: ContextVar['TaskGroup'] = ContextVar('current_taskgroup')
+current_taskgroup: ContextVar[TaskGroup] = ContextVar("current_taskgroup")
 
 
 class TaskGroup(asyncio.TaskGroup):
+    _name: str
+    _current_taskgroup_token: Token[TaskGroup]
 
-    def __init__(self, *, name=None):
+    def __init__(self, *, name: str | None = None) -> None:
         super().__init__()
         if name is None:
             self._name = f"tg-{_name_counter()}"
         else:
             self._name = str(name)
 
-    def get_name(self):
+    def get_name(self) -> str:
         return self._name
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> Self:
         self._current_taskgroup_token = current_taskgroup.set(self)
         return await super().__aenter__()
 
-    async def __aexit__(self, et, exc, tb):
+    async def __aexit__(
+        self,
+        et: type[BaseException] | None,
+        exc: BaseException | None,
+        tb: TracebackType | None,
+    ) -> None:
         try:
-            return await super().__aexit__(et, exc, tb)
+            await super().__aexit__(et, exc, tb)
         except BaseExceptionGroup as eg:
             # Just wrap the exception group as TaskGroupError for backward
             # compatibility.  In Python 3.11 or higher, TaskGroupError
@@ -42,4 +55,4 @@ class TaskGroup(asyncio.TaskGroup):
             current_taskgroup.reset(self._current_taskgroup_token)
 
 
-_name_counter = itertools.count(1).__next__
+_name_counter: Callable[[], int] = itertools.count(1).__next__
