@@ -1,73 +1,85 @@
+from __future__ import annotations
+
 import asyncio
 import itertools
 import logging
 import sys
 import traceback
+import weakref
+from collections.abc import Awaitable, Callable, Sequence
 from contextvars import ContextVar, Token
 from types import TracebackType
 from typing import (
     Any,
-    Awaitable,
-    Callable,
-    Coroutine,
-    List,
-    Optional,
-    Sequence,
-    Type,
-    Union,
+    Self,
+    TypeVar,
+    cast,
 )
-import weakref
+
+from typing_extensions import deprecated
+
+from aiotools.types import CoroutineLike
 
 from .. import compat
 from .types import AsyncExceptionHandler
 
 __all__ = (
-    'PersistentTaskGroup',
-    'current_ptaskgroup',
+    "PersistentTaskGroup",
+    "current_ptaskgroup",
 )
 
-current_ptaskgroup: ContextVar['PersistentTaskGroup'] = \
-    ContextVar('current_ptaskgroup')
+current_ptaskgroup: ContextVar[PersistentTaskGroup] = ContextVar("current_ptaskgroup")  # type: ignore[deprecated]
 
 _ptaskgroup_idx = itertools.count()
-_log = logging.getLogger(__name__)
-_all_ptaskgroups: weakref.WeakSet['PersistentTaskGroup'] = weakref.WeakSet()
+_log = logging.getLogger(__spec__.name)
+_all_ptaskgroups: weakref.WeakSet[PersistentTaskGroup] = weakref.WeakSet()  # type: ignore[deprecated]
+
+T = TypeVar("T")
 
 
-async def _default_exc_handler(exc_type, exc_obj, exc_tb) -> None:
+async def _default_exc_handler(
+    exc_type: type[BaseException],
+    exc_obj: BaseException,
+    exc_tb: TracebackType,
+) -> None:
     traceback.print_exc()
 
 
+@deprecated("Use aiotools.TaskScope instead.")
 class PersistentTaskGroup:
-
-    _base_error: Optional[BaseException]
+    _base_error: BaseException | None
     _exc_handler: AsyncExceptionHandler
-    _errors: Optional[List[BaseException]]
-    _tasks: "weakref.WeakSet[asyncio.Task]"
-    _on_completed_fut: Optional[asyncio.Future]
-    _current_taskgroup_token: Optional[Token["PersistentTaskGroup"]]
+    _tasks: set[asyncio.Task[Any]]
+    _on_completed_fut: asyncio.Future[Any] | None
+    _current_taskgroup_token: Token[PersistentTaskGroup] | None
+    _entered: bool
+    _exiting: bool
+    _aborting: bool
+    _name: str
+    _parent_cancel_requested: bool
+    _unfinished_tasks: int
+    _parent_task: asyncio.Task[Any] | None
 
     @classmethod
-    def all_ptaskgroups(cls) -> Sequence['PersistentTaskGroup']:
+    def all_ptaskgroups(cls) -> Sequence[PersistentTaskGroup]:
         return list(_all_ptaskgroups)
 
     def __init__(
         self,
         *,
-        name: str = None,
-        exception_handler: AsyncExceptionHandler = None,
+        name: str | None = None,
+        exception_handler: AsyncExceptionHandler | None = None,
     ) -> None:
         self._entered = False
         self._exiting = False
         self._aborting = False
-        self._errors = []
         self._base_error = None
         self._name = name or f"{next(_ptaskgroup_idx)}"
         self._parent_cancel_requested = False
         self._unfinished_tasks = 0
         self._on_completed_fut = None
         self._parent_task = compat.current_task()
-        self._tasks = weakref.WeakSet()
+        self._tasks = set()
         self._current_taskgroup_token = None
         _all_ptaskgroups.add(self)
         if exception_handler is None:
@@ -83,10 +95,10 @@ class PersistentTaskGroup:
 
     def create_task(
         self,
-        coro: Coroutine[Any, Any, Any],
+        coro: CoroutineLike[T],
         *,
-        name: str = None,
-    ) -> Awaitable[Any]:
+        name: str | None = None,
+    ) -> Awaitable[T]:
         if not self._entered:
             # When used as object attribute, auto-enter.
             self._entered = True
@@ -96,11 +108,11 @@ class PersistentTaskGroup:
 
     def _create_task_with_name(
         self,
-        coro: Coroutine[Any, Any, Any],
+        coro: CoroutineLike[T],
         *,
-        name: str = None,
-        cb: Callable[[asyncio.Task], Any],
-    ) -> Awaitable[Any]:
+        name: str | None = None,
+        cb: Callable[[asyncio.Task[T | None]], None],
+    ) -> Awaitable[T]:
         loop = compat.get_running_loop()
         result_future = loop.create_future()
         child_task = loop.create_task(
@@ -111,15 +123,16 @@ class PersistentTaskGroup:
         self._unfinished_tasks += 1
         child_task.add_done_callback(cb)
         self._tasks.add(child_task)
+        child_task.add_done_callback(self._tasks.discard)
         return result_future
 
     def _is_base_error(self, exc: BaseException) -> bool:
         assert isinstance(exc, BaseException)
         return isinstance(exc, (SystemExit, KeyboardInterrupt))
 
-    async def _wait_completion(self) -> Optional[BaseException]:
+    async def _wait_completion(self) -> BaseException | None:
         loop = compat.get_running_loop()
-        propagate_cancellation_error = None
+        propagate_cancellation_error: BaseException | None = None
         while self._unfinished_tasks:
             if self._on_completed_fut is None:
                 self._on_completed_fut = loop.create_future()
@@ -148,14 +161,14 @@ class PersistentTaskGroup:
 
     async def _task_wrapper(
         self,
-        coro: Coroutine,
-        result_future: weakref.ref[asyncio.Future],
-    ) -> Any:
+        coro: CoroutineLike[T],
+        result_future: weakref.ref[asyncio.Future[T]],
+    ) -> T | None:
         loop = compat.get_running_loop()
         task = compat.current_task()
         fut = result_future()
         try:
-            ret = await coro
+            ret = await cast(Awaitable[T], coro)  # forced cast for Python 3.12 or older
             if fut is not None:
                 fut.set_result(ret)
             return ret
@@ -173,26 +186,32 @@ class PersistentTaskGroup:
             try:
                 if fut is not None:
                     fut.set_exception(e)
-                await self._exc_handler(*sys.exc_info())
+                exc_info = cast(
+                    tuple[type[BaseException], BaseException, TracebackType],
+                    sys.exc_info(),
+                )
+                await self._exc_handler(*exc_info)
             except Exception as exc:
                 # If there are exceptions inside the exception handler
                 # we report it as soon as possible using the event loop's
                 # exception handler, instead of postponing
                 # to the timing when PersistentTaskGroup terminates.
                 loop.call_exception_handler({
-                    'message': f"Got an unhandled exception "
-                               f"in the exception handler of Task {task!r}",
-                    'exception': exc,
-                    'task': task,
+                    "message": (
+                        "Got an unhandled exception "
+                        f"in the exception handler of Task {task!r}"
+                    ),
+                    "exception": exc,
+                    "task": task,
                 })
+            return None
         finally:
             del fut
 
-    def _on_task_done(self, task: asyncio.Task) -> None:
+    def _on_task_done(self, task: asyncio.Task[Any]) -> None:
         self._unfinished_tasks -= 1
         assert self._unfinished_tasks >= 0
         assert self._parent_task is not None
-        assert self._errors is not None
 
         if self._on_completed_fut is not None and not self._unfinished_tasks:
             if not self._on_completed_fut.done():
@@ -207,7 +226,6 @@ class PersistentTaskGroup:
             return
 
         # Now the exception is BaseException.
-        self._errors.append(exc)
         if self._base_error is None:
             self._base_error = exc
 
@@ -215,7 +233,7 @@ class PersistentTaskGroup:
         if not self._parent_task.cancelling():
             self._parent_cancel_requested = True
 
-    async def __aenter__(self) -> "PersistentTaskGroup":
+    async def __aenter__(self) -> Self:
         self._parent_task = compat.current_task()
         self._current_taskgroup_token = current_ptaskgroup.set(self)
         self._entered = True
@@ -223,34 +241,28 @@ class PersistentTaskGroup:
 
     async def __aexit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
-    ) -> Optional[bool]:
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> bool | None:
+        assert self._parent_task is not None
         self._exiting = True
-        assert self._errors is not None
-        propagate_cancellation_error: Optional[
-            Union[Type[BaseException], BaseException]
-        ] = None
-
-        if (exc_val is not None and
-                self._is_base_error(exc_val) and
-                self._base_error is None):
-            self._base_error = exc_val
+        propagate_cancellation_error: type[BaseException] | BaseException | None = None
 
         if (
-            exc_type is asyncio.CancelledError or
-            exc_type is asyncio.TimeoutError
+            exc_val is not None
+            and self._is_base_error(exc_val)
+            and self._base_error is None
         ):
+            self._base_error = exc_val
+
+        if exc_type is asyncio.CancelledError or exc_type is asyncio.TimeoutError:
             if self._parent_cancel_requested:
                 self._parent_task.uncancel()
             else:
                 propagate_cancellation_error = exc_type
         if exc_type is not None and not self._aborting:
-            if (
-                exc_type is asyncio.CancelledError or
-                exc_type is asyncio.TimeoutError
-            ):
+            if exc_type is asyncio.CancelledError or exc_type is asyncio.TimeoutError:
                 propagate_cancellation_error = exc_type
             self._trigger_shutdown()
 
@@ -263,38 +275,19 @@ class PersistentTaskGroup:
         if propagate_cancellation_error is not None:
             raise propagate_cancellation_error
 
-        if exc_val is not None and exc_type is not asyncio.CancelledError:
-            # If there are any unhandled errors, let's add them to
-            # the bubbled up exception group.
-            # Normally, they should have been swallowed and logged
-            # by the fallback exception handler.
-            self._errors.append(exc_val)
-
-        if self._errors:
-            # Bubble up errors
-            errors = self._errors
-            self._errors = None
-            me = BaseExceptionGroup(
-                'unhandled errors in a PersistentTaskGroup',
-                errors,
-            )
-            raise me from None
-
         return None
 
     def __repr__(self) -> str:
-        info = ['']
+        info = [""]
         if self._name:
-            info.append(f'name={self._name}')
+            info.append(f"name={self._name}")
         if self._tasks:
-            info.append(f'tasks={len(self._tasks)}')
+            info.append(f"tasks={len(self._tasks)}")
         if self._unfinished_tasks:
-            info.append(f'unfinished={self._unfinished_tasks}')
-        if self._errors:
-            info.append(f'errors={len(self._errors)}')
+            info.append(f"unfinished={self._unfinished_tasks}")
         if self._aborting:
-            info.append('cancelling')
+            info.append("cancelling")
         elif self._entered:
-            info.append('entered')
-        info_str = ' '.join(info)
-        return f'<PersistentTaskGroup({info_str})>'
+            info.append("entered")
+        info_str = " ".join(info)
+        return f"<PersistentTaskGroup({info_str})>"
