@@ -1,28 +1,81 @@
+from __future__ import annotations
+
 import asyncio
 import sys
 import warnings
+from collections.abc import AsyncGenerator, AsyncIterator
+from contextlib import suppress
+from contextvars import ContextVar
 
-import aiotools
-from aiotools.context import AbstractAsyncContextManager
 import pytest
 
+import aiotools
+from aiotools.context import (
+    AbstractAsyncContextManager,
+    resetting,
+)
 
-def test_actxmgr_types():
+my_variable: ContextVar[int] = ContextVar("my_variable")
+
+
+def test_resetting_ctxvar() -> None:
+    with pytest.raises(LookupError):
+        my_variable.get()
+    with resetting(my_variable, 1):
+        assert my_variable.get() == 1
+        with resetting(my_variable, 2):
+            assert my_variable.get() == 2
+        assert my_variable.get() == 1
+    with pytest.raises(LookupError):
+        my_variable.get()
+
+    # should behave the same way even when an exception occurs
+    with suppress(RuntimeError):
+        with resetting(my_variable, 10):
+            assert my_variable.get() == 10
+            raise RuntimeError("oops")
+    with pytest.raises(LookupError):
+        my_variable.get()
+
+
+@pytest.mark.asyncio
+async def test_resetting_ctxvar_async() -> None:
+    with pytest.raises(LookupError):
+        my_variable.get()
+    async with resetting(my_variable, 1):
+        assert my_variable.get() == 1
+        async with resetting(my_variable, 2):
+            assert my_variable.get() == 2
+        assert my_variable.get() == 1
+    with pytest.raises(LookupError):
+        my_variable.get()
+
+    # should behave the same way even when an exception occurs
+    with suppress(RuntimeError):
+        async with resetting(my_variable, 10):
+            assert my_variable.get() == 10
+            raise RuntimeError("oops")
+    with pytest.raises(LookupError):
+        my_variable.get()
+
+
+def test_actxmgr_types() -> None:
+    from typing import Any
 
     assert issubclass(aiotools.AsyncContextManager, AbstractAsyncContextManager)
 
-    class boilerplate_ctx():
-        async def __aenter__(self):
+    class boilerplate_ctx:
+        async def __aenter__(self) -> Any:
             return self
 
-        async def __aexit__(self, *exc_info):
+        async def __aexit__(self, *exc_info: Any) -> None:
             return None
 
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx() -> AsyncIterator[None]:
         yield
 
-    async def simple_agen():
+    async def simple_agen() -> AsyncGenerator[None]:
         yield
 
     assert issubclass(boilerplate_ctx, AbstractAsyncContextManager)
@@ -31,12 +84,11 @@ def test_actxmgr_types():
 
 
 @pytest.mark.asyncio
-async def test_actxmgr(event_loop):
-
+async def test_actxmgr() -> None:
     step = 0
 
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         nonlocal step
         step = 1
         await asyncio.sleep(0)
@@ -49,41 +101,41 @@ async def test_actxmgr(event_loop):
             step = 4
 
     step = 0
-    async with simple_ctx('hello') as msg:
+    async with simple_ctx("hello") as msg:
         assert step == 2
-        assert msg == 'hello'
+        assert msg == "hello"
     assert step == 4
 
     step = 0
     try:
-        async with simple_ctx('world') as msg:
+        async with simple_ctx("world") as msg:
             assert step == 2
-            assert msg == 'world'
+            assert msg == "world"
             await asyncio.sleep(0)
-            raise ValueError('something wrong')
+            raise ValueError("something wrong")
     except Exception as e:
         await asyncio.sleep(0)
-        assert e.args[0] == 'something wrong'
+        assert e.args[0] == "something wrong"
         assert step == 4
 
 
-@pytest.mark.skipif(sys.version_info >= (3, 7, 0),
-                    reason='Deprecated in Python 3.7 or higher')
+@pytest.mark.skipif(
+    sys.version_info >= (3, 7, 0), reason="Deprecated in Python 3.7 or higher"
+)
 @pytest.mark.asyncio
-async def test_actxmgr_reuse(event_loop):
-
+async def test_actxmgr_reuse() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         yield msg
 
-    cm = simple_ctx('hello')
+    cm = simple_ctx("hello")
 
     async with cm as msg:
-        assert msg == 'hello'
+        assert msg == "hello"
 
     try:
         async with cm as msg:
-            assert msg == 'hello'
+            assert msg == "hello"
     except BaseException as exc:
         assert isinstance(exc, RuntimeError)
         assert "didn't yield" in exc.args[0]
@@ -91,52 +143,53 @@ async def test_actxmgr_reuse(event_loop):
     cm = cm._recreate_cm()
 
     async with cm as msg:
-        assert msg == 'hello'
+        assert msg == "hello"
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_in_context_body():
-
+async def test_actxmgr_exception_in_context_body() -> None:
     # Exceptions raised in the context body
     # should be transparently raised.
 
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         await asyncio.sleep(0)
         yield msg
         await asyncio.sleep(0)
 
     with pytest.raises(ZeroDivisionError):
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx("hello") as msg:
+            assert msg == "hello"
             raise ZeroDivisionError
 
+    exc = RuntimeError("oops")
     try:
-        exc = RuntimeError('oops')
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx("hello") as msg:
+            assert msg == "hello"
             raise exc
     except BaseException as e:
         assert e is exc
-        assert e.args[0] == 'oops'
+        assert e.args[0] == "oops"
     else:
         pytest.fail()
 
-    cm = simple_ctx('hello')
-    ret = await cm.__aenter__()
-    assert ret == 'hello'
-    ret = await cm.__aexit__(ValueError, None, None)
-    assert not ret
+    cm = simple_ctx("hello")
+    ret_enter = await cm.__aenter__()
+    assert ret_enter == "hello"
+    try:
+        raise ValueError()
+    except ValueError:
+        ret_exit = await cm.__aexit__(*sys.exc_info())
+        assert not ret_exit
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_in_initialization():
-
+async def test_actxmgr_exception_in_initialization() -> None:
     # Any exception before first yield is just transparently
     # raised out to the context block.
 
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx1(msg: str) -> AsyncIterator[str]:
         await asyncio.sleep(0)
         raise ZeroDivisionError
         await asyncio.sleep(0)
@@ -144,17 +197,17 @@ async def test_actxmgr_exception_in_initialization():
         await asyncio.sleep(0)
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx1("hello") as msg:
+            assert msg == "hello"
     except ZeroDivisionError:
         pass
     else:
         pytest.fail()
 
-    exc = RuntimeError('oops')
+    exc = RuntimeError("oops")
 
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx2(msg: str) -> AsyncIterator[str]:
         await asyncio.sleep(0)
         raise exc
         await asyncio.sleep(0)
@@ -162,20 +215,19 @@ async def test_actxmgr_exception_in_initialization():
         await asyncio.sleep(0)
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx2("hello") as msg:
+            assert msg == "hello"
     except BaseException as e:
         assert e is exc
-        assert e.args[0] == 'oops'
+        assert e.args[0] == "oops"
     else:
         pytest.fail()
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_in_finalization():
-
+async def test_actxmgr_exception_in_finalization() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx1(msg: str) -> AsyncIterator[str]:
         await asyncio.sleep(0)
         yield msg
         await asyncio.sleep(0)
@@ -183,139 +235,134 @@ async def test_actxmgr_exception_in_finalization():
         await asyncio.sleep(0)
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx1("hello") as msg:
+            assert msg == "hello"
     except ZeroDivisionError:
         pass
     else:
         pytest.fail()
 
-    exc = RuntimeError('oops')
+    exc = RuntimeError("oops")
 
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx2(msg: str) -> AsyncIterator[str]:
         yield msg
         raise exc
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx2("hello") as msg:
+            assert msg == "hello"
     except BaseException as e:
         assert e is exc
-        assert e.args[0] == 'oops'
+        assert e.args[0] == "oops"
     else:
         pytest.fail()
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_uncaught():
-
+async def test_actxmgr_exception_uncaught() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         await asyncio.sleep(0)
         yield msg
         await asyncio.sleep(0)
 
     try:
-        async with simple_ctx('hello'):
-            raise IndexError('bomb')
+        async with simple_ctx("hello"):
+            raise IndexError("bomb")
     except BaseException as e:
         assert isinstance(e, IndexError)
-        assert e.args[0] == 'bomb'
+        assert e.args[0] == "bomb"
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_nested():
-
+async def test_actxmgr_exception_nested() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         yield msg
 
     try:
-        async with simple_ctx('hello') as msg1:
-            async with simple_ctx('world') as msg2:
-                assert msg1 == 'hello'
-                assert msg2 == 'world'
-                raise IndexError('bomb1')
+        async with simple_ctx("hello") as msg1:
+            async with simple_ctx("world") as msg2:
+                assert msg1 == "hello"
+                assert msg2 == "world"
+                raise IndexError("bomb1")
     except BaseException as exc:
         assert isinstance(exc, IndexError)
-        assert 'bomb1' == exc.args[0]
+        assert "bomb1" == exc.args[0]
     else:
         pytest.fail()
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_chained():
-
+async def test_actxmgr_exception_chained() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         try:
             await asyncio.sleep(0)
             yield msg
         except Exception as e:
             await asyncio.sleep(0)
             # exception is chained
-            raise ValueError('bomb2') from e
+            raise ValueError("bomb2") from e
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
-            raise IndexError('bomb1')
+        async with simple_ctx("hello") as msg:
+            assert msg == "hello"
+            raise IndexError("bomb1")
     except BaseException as exc:
         assert isinstance(exc, ValueError)
-        assert 'bomb2' == exc.args[0]
+        assert "bomb2" == exc.args[0]
         assert isinstance(exc.__cause__, IndexError)
-        assert 'bomb1' == exc.__cause__.args[0]
+        assert "bomb1" == exc.__cause__.args[0]
     else:
         pytest.fail()
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_exception_replaced():
-
+async def test_actxmgr_exception_replaced() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx(msg: str) -> AsyncIterator[str]:
         try:
             await asyncio.sleep(0)
             yield msg
         except Exception:
             await asyncio.sleep(0)
             # exception is replaced
-            raise ValueError('bomb2')
+            raise ValueError("bomb2")
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
-            raise IndexError('bomb1')
+        async with simple_ctx("hello") as msg:
+            assert msg == "hello"
+            raise IndexError("bomb1")
     except BaseException as exc:
         assert isinstance(exc, ValueError)
-        assert 'bomb2' == exc.args[0]
+        assert "bomb2" == exc.args[0]
         assert exc.__cause__ is None
     else:
         pytest.fail()
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_stopaiter(event_loop):
-
+async def test_actxmgr_stopaiter() -> None:
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx1() -> AsyncIterator[int]:
         await asyncio.sleep(0)
         yield 1
         await asyncio.sleep(0)
         yield 2
         await asyncio.sleep(0)
 
-    cm = simple_ctx()
-    ret = await cm.__aenter__()
-    assert ret == 1
-    ret = await cm.__aexit__(StopAsyncIteration, None, None)
-    assert ret is False
+    cm = simple_ctx1()
+    ret_enter = await cm.__aenter__()
+    assert ret_enter == 1
+    ret_exit = await cm.__aexit__(StopAsyncIteration, None, None)
+    assert ret_exit is False
 
     step = 0
 
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx2() -> AsyncIterator[None]:
         nonlocal step
         step = 1
         try:
@@ -324,11 +371,11 @@ async def test_actxmgr_stopaiter(event_loop):
         finally:
             step = 2
             await asyncio.sleep(0)
-            raise StopAsyncIteration('x')
+            raise StopAsyncIteration("x")
 
     try:
         step = 0
-        async with simple_ctx():
+        async with simple_ctx2():
             assert step == 1
     except RuntimeError:
         # converted to RuntimeError
@@ -338,12 +385,11 @@ async def test_actxmgr_stopaiter(event_loop):
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_transparency(event_loop):
-
+async def test_actxmgr_transparency() -> None:
     step = 0
 
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx1() -> AsyncIterator[None]:
         nonlocal step
         step = 1
         await asyncio.sleep(0)
@@ -351,10 +397,10 @@ async def test_actxmgr_transparency(event_loop):
         step = 2
         await asyncio.sleep(0)
 
+    exc = StopAsyncIteration("x")
     try:
-        exc = StopAsyncIteration('x')
         step = 0
-        async with simple_ctx():
+        async with simple_ctx1():
             assert step == 1
             raise exc
     except StopAsyncIteration as e:
@@ -365,7 +411,7 @@ async def test_actxmgr_transparency(event_loop):
         pytest.fail()
 
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx2() -> AsyncIterator[None]:
         nonlocal step
         step = 1
         await asyncio.sleep(0)
@@ -375,7 +421,7 @@ async def test_actxmgr_transparency(event_loop):
 
     try:
         step = 0
-        async with simple_ctx():
+        async with simple_ctx2():
             assert step == 1
             raise ValueError
     except ValueError:
@@ -385,7 +431,7 @@ async def test_actxmgr_transparency(event_loop):
         pytest.fail()
 
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx3() -> AsyncIterator[None]:
         nonlocal step
         step = 1
         await asyncio.sleep(0)
@@ -395,10 +441,10 @@ async def test_actxmgr_transparency(event_loop):
             step = 2
             await asyncio.sleep(0)
 
+    exc = StopAsyncIteration("x")
     try:
-        exc = StopAsyncIteration('x')
         step = 0
-        async with simple_ctx():
+        async with simple_ctx3():
             assert step == 1
             raise exc
     except StopAsyncIteration as e:
@@ -410,7 +456,7 @@ async def test_actxmgr_transparency(event_loop):
         pytest.fail()
 
     @aiotools.actxmgr
-    async def simple_ctx():
+    async def simple_ctx4() -> AsyncIterator[None]:
         nonlocal step
         step = 1
         await asyncio.sleep(0)
@@ -422,7 +468,7 @@ async def test_actxmgr_transparency(event_loop):
 
     try:
         step = 0
-        async with simple_ctx():
+        async with simple_ctx4():
             assert step == 1
             raise ValueError
     except ValueError:
@@ -434,10 +480,9 @@ async def test_actxmgr_transparency(event_loop):
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_no_stop(event_loop):
-
+async def test_actxmgr_no_stop() -> None:
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx1(msg: str) -> AsyncIterator[str]:
         await asyncio.sleep(0)
         yield msg
         await asyncio.sleep(0)
@@ -445,24 +490,24 @@ async def test_actxmgr_no_stop(event_loop):
         await asyncio.sleep(0)
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
+        async with simple_ctx1("hello") as msg:
+            assert msg == "hello"
     except RuntimeError as exc:
         assert "didn't stop" in exc.args[0]
     else:
         pytest.fail()
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
-            raise ValueError('oops')
+        async with simple_ctx1("hello") as msg:
+            assert msg == "hello"
+            raise ValueError("oops")
     except ValueError as exc:
-        assert exc.args[0] == 'oops'
+        assert exc.args[0] == "oops"
     else:
         pytest.fail()
 
     @aiotools.actxmgr
-    async def simple_ctx(msg):
+    async def simple_ctx2(msg: str) -> AsyncIterator[str]:
         try:
             await asyncio.sleep(0)
             yield msg
@@ -471,9 +516,9 @@ async def test_actxmgr_no_stop(event_loop):
             yield msg
 
     try:
-        async with simple_ctx('hello') as msg:
-            assert msg == 'hello'
-            raise ValueError('oops')
+        async with simple_ctx2("hello") as msg:
+            assert msg == "hello"
+            raise ValueError("oops")
     except RuntimeError as exc:
         assert "didn't stop after" in exc.args[0]
     else:
@@ -481,17 +526,16 @@ async def test_actxmgr_no_stop(event_loop):
 
 
 @pytest.mark.asyncio
-async def test_actxmgr_no_yield(event_loop):
-
-    @aiotools.actxmgr
-    async def no_yield_ctx1(msg):
+async def test_actxmgr_no_yield() -> None:
+    @aiotools.actxmgr  # type: ignore[arg-type]
+    async def no_yield_ctx1(msg: str) -> AsyncIterator[None]:  # type: ignore[empty-body]
         pass
 
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
 
         try:
-            async with no_yield_ctx1('hello'):
+            async with no_yield_ctx1("hello"):
                 pass
         except RuntimeError as exc:
             assert "must be an async-gen" in exc.args[0]
@@ -503,123 +547,17 @@ async def test_actxmgr_no_yield(event_loop):
             pytest.fail()
 
 
-@pytest.mark.skipif(sys.version_info >= (3, 7, 0),
-                    reason='Deprecated in Python 3.7 or higher')
 @pytest.mark.asyncio
-async def test_actxdecorator(event_loop):
-
-    step = 0
-
-    class myacontext(aiotools.AsyncContextDecorator):
-
-        async def __aenter__(self):
-            nonlocal step
-            step = 1
-            return self
-
-        async def __aexit__(self, exc_type, exc, tb):
-            nonlocal step
-            step = 2
-            return False
-
-    # NOTE: don't forget the ending parenthesis which instantiates myacontext
-    @myacontext()
-    async def myfunc():
-        assert step == 1
-
-    assert step == 0
-    await myfunc()
-    assert step == 2
-
-    @myacontext()
-    async def myfunc():
-        assert step == 1
-        raise RuntimeError('oops')
-
-    step = 0
-    assert step == 0
-    try:
-        await myfunc()
-    except BaseException as e:
-        assert e.args[0] == 'oops'
-        assert step == 2
-    finally:
-        assert step == 2
-
-    @myacontext()
-    async def myfunc():
-        assert step == 1
-
-    # Use as a plain async context manager.
-    step = 0
-    assert step == 0
-    async with myacontext():
-        assert step == 1
-    assert step == 2
-
-
-@pytest.mark.asyncio
-async def test_actxgroup(event_loop):
-
-    # Test basic function.
-    exit_count = 0
-
+async def test_actxgroup_exception_from_cm() -> None:
     @aiotools.actxmgr
-    async def ctx(a):
-        nonlocal exit_count
-        await asyncio.sleep(0)
-        yield a + 10
-        await asyncio.sleep(0)
-        exit_count += 1
-
-    ctxgrp = aiotools.actxgroup()
-
-    for i in range(3):
-        ctxgrp.add(ctx(i))
-
-    async with ctxgrp as values:
-        assert values[0] == 10
-        assert values[1] == 11
-        assert values[2] == 12
-        assert len(ctxgrp._cm_yields) == 3
-
-    assert exit_count == 3
-    assert len(ctxgrp._cm_yields) == 0
-    returns = ctxgrp.exit_states()
-    assert not returns[0]
-    assert not returns[1]
-    assert not returns[2]
-
-    # Test generator/iterator initialization
-    exit_count = 0
-    ctxgrp = aiotools.actxgroup(ctx(i) for i in range(3))
-
-    async with ctxgrp as values:
-        assert values[0] == 10
-        assert values[1] == 11
-        assert values[2] == 12
-        assert len(ctxgrp._cm_yields) == 3
-
-    assert exit_count == 3
-    assert len(ctxgrp._cm_yields) == 0
-    returns = ctxgrp.exit_states()
-    assert not returns[0]
-    assert not returns[1]
-    assert not returns[2]
-
-
-@pytest.mark.asyncio
-async def test_actxgroup_exception_from_cm(event_loop):
-
-    @aiotools.actxmgr
-    async def ctx1(a):
+    async def ctx1(a: int) -> AsyncIterator[int]:
         await asyncio.sleep(0)
         raise asyncio.CancelledError
         await asyncio.sleep(0)
         yield a
 
     @aiotools.actxmgr
-    async def ctx2(a):
+    async def ctx2(a: int) -> AsyncIterator[int]:
         await asyncio.sleep(0)
         raise ZeroDivisionError
         await asyncio.sleep(0)
@@ -632,12 +570,12 @@ async def test_actxgroup_exception_from_cm(event_loop):
         assert isinstance(values[1], ZeroDivisionError)
 
     @aiotools.actxmgr
-    async def ctx3(a):
+    async def ctx3(a: int) -> AsyncIterator[int]:
         yield a
         raise asyncio.CancelledError
 
     @aiotools.actxmgr
-    async def ctx4(a):
+    async def ctx4(a: int) -> AsyncIterator[int]:
         yield a
         raise ZeroDivisionError
 
@@ -653,12 +591,11 @@ async def test_actxgroup_exception_from_cm(event_loop):
 
 
 @pytest.mark.asyncio
-async def test_actxgroup_exception_from_body(event_loop):
-
+async def test_actxgroup_exception_from_body() -> None:
     exit_count = 0
 
     @aiotools.actxmgr
-    async def ctx(a):
+    async def ctx1(a: int) -> AsyncIterator[int]:
         nonlocal exit_count
         await asyncio.sleep(0)
         yield a
@@ -667,7 +604,7 @@ async def test_actxgroup_exception_from_body(event_loop):
         # If not handled, finalization will not be executed.
         exit_count += 1
 
-    ctxgrp = aiotools.actxgroup([ctx(1), ctx(2)])
+    ctxgrp = aiotools.actxgroup([ctx1(1), ctx1(2)])
 
     try:
         async with ctxgrp as values:
@@ -680,12 +617,12 @@ async def test_actxgroup_exception_from_body(event_loop):
     exits = ctxgrp.exit_states()
     assert not exits[0]  # __aexit__ are called successfully
     assert not exits[1]
-    assert exit_count == 0   # but they errored internally
+    assert exit_count == 0  # but they errored internally
 
     exit_count = 0
 
     @aiotools.actxmgr
-    async def ctx(a):
+    async def ctx2(a: int) -> AsyncIterator[int]:
         nonlocal exit_count
         try:
             await asyncio.sleep(0)
@@ -695,7 +632,7 @@ async def test_actxgroup_exception_from_body(event_loop):
             # Ensure finalization is executed.
             exit_count += 1
 
-    ctxgrp = aiotools.actxgroup([ctx(1), ctx(2)])
+    ctxgrp = aiotools.actxgroup([ctx2(1), ctx2(2)])
 
     try:
         async with ctxgrp as values:
@@ -708,15 +645,14 @@ async def test_actxgroup_exception_from_body(event_loop):
     exits = ctxgrp.exit_states()
     assert not exits[0]  # __aexit__ are called successfully
     assert not exits[1]
-    assert exit_count == 2   # they also suceeeded
+    assert exit_count == 2  # they also suceeeded
 
 
 @pytest.mark.asyncio
-async def test_aclosing(event_loop):
-
+async def test_aclosing() -> None:
     finalized = False
 
-    async def myiter():
+    async def myiter() -> AsyncGenerator[int]:
         nonlocal finalized
         try:
             yield 1
@@ -732,7 +668,7 @@ async def test_aclosing(event_loop):
     mysum = 0
     detect_cancellation = False
 
-    async def mytask():
+    async def mytask() -> None:
         nonlocal mysum, detect_cancellation
         try:
             async with aiotools.aclosing(myiter()) as agen:
@@ -741,7 +677,7 @@ async def test_aclosing(event_loop):
         except asyncio.CancelledError:
             detect_cancellation = True
 
-    t = event_loop.create_task(mytask())
+    t = asyncio.create_task(mytask())
     await asyncio.sleep(0.3)
     t.cancel()
     await t
